"""
WSGI config for cpanel project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/3.0/howto/deployment/wsgi/
"""

import os

from django.core.wsgi import get_wsgi_application

try:
    import empty_my_fridge.manage
    settings = 'empty_my_fridge.cpanel.settings'
except ModuleNotFoundError:
    settings = 'cpanel.settings'

os.environ.setdefault('DJANGO_SETTINGS_MODULE', settings)

application = get_wsgi_application()



from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.shortcuts import render, redirect, HttpResponseRedirect
import pyrebase
from django.views.decorators.csrf import csrf_exempt
from django.contrib import messages
from datetime import date
from django.views.decorators.cache import never_cache
from django.contrib import auth
from django.contrib.auth.decorators import login_required
import json
import sys
sys.path.append('..')
sys.path.append('empty_my_fridge/model')
sys.path.append('empty_my_fridge/empty_my_fridge/model')
import time
from category import Category
from route import ActivityPage
from message import Message
from recipes import Recipes
from user import User
import config
from . import allrecipes




firebase = pyrebase.initialize_app(config.myConfig())

auth_fb = firebase.auth()
fb_storage = firebase.storage()
db = firebase.database()
m_user = User()
m_message = Message()
m_activity = ActivityPage()
m_category = Category()
recipes = Recipes(db, m_user, allrecipes)
recipes._get_all_recipes_()

# Home Page


@csrf_exempt
def home(request):
    m_category.set_filter_list(None)
    recipes.set_filter_list(None)
    m_category.set_sorting_type("name_A")
    recipes.set_sorting_type("name_A")
    recipes.set_fridge_sorting_type("name_A")
    recipes.set_fridge_recipes(None)
    if recipes.get_scraped():
        recipes._get_all_recipes_()
        recipes.set_scraped(False)
    if m_user._isNone_():
        return render(request, 'home.html')
    else:
        user = m_user._getUser_()
        data = {
            "user": user,
        }
        return render(request, 'home.html', {"data": data})


def Fridge_matches(all_recipes):
    possible_recipes = []
    partial_recipes = []
    fridge_ingredients = db.child("users").child(m_user._getUser_Id_()).child("Fridge").get().val()
    for recipe in all_recipes:
        #recipe_details = recipe.val()
        try:
            recipe_ingredients = recipe["recipe_ingredients"]
            r_i = set(recipe_ingredients)
            if(fridge_ingredients):
                f_i = set(fridge_ingredients)
            else:
                f_i = set()
            if r_i.issubset(f_i):
                possible_recipes.append(recipe)
            elif(len(missing:=(r_i-f_i))<4):
                recipe["missing_ingredients"]=list(missing)
                partial_recipes.append(recipe)
        except KeyError:
            pass
    return({"exact":possible_recipes,"partial":partial_recipes})

def sort_recipes(recipe_list, type):
    result = None
    if type == "name_A":
        result = sorted(
            recipe_list, key=lambda x: x["recipe_name"].title(), reverse=False)
    elif type == "name_D":
        result = sorted(
            recipe_list, key=lambda x: x["recipe_name"].title(), reverse=True)
    elif type == "fav_A":
        result = sorted(recipe_list, key=lambda x: x["likes"], reverse=False)
    elif type == "fav_D":
        result = sorted(recipe_list, key=lambda x: x["likes"], reverse=True)
    elif type == "mis_A":
        result = sorted(recipe_list, key=lambda x: len(x["missing_ingredients"]), reverse=False)
    elif type == "mis_D":
        result = sorted(recipe_list, key=lambda x: len(x["missing_ingredients"]), reverse=True)
    return result


@csrf_exempt
def scrape_page(request):
    if m_user._isNone_():
        return HttpResponseRedirect("/empty_my_fridge/login/")
    else:
        uid = m_user._getUser_Id_()
        isAdmin = False
        report = "Your administrative privileges have been verified\n\nScraping...Please wait"
        admins = db.child("admin").child(
            "UPLwshBH98OmbVivV").child("scrapers").get().val()
        for admin in admins:
            if str(admin) == str(uid):
                isAdmin = True
                break
        if isAdmin:
            print(report)
            start = time.time()
            allrecipes.allrecipes(db)
            end = time.time()
            total_time = end - start
            minutes = 0
            while total_time >= 0:
                if total_time - 60 >= 0:
                    minutes += 1
                total_time -= 60

            seconds = total_time + 60
            report = "Finished scraping in {0} minute(s) and {1:0.1f} seconds.".format(
                minutes, seconds)
            recipes.set_scraped(True)
        else:
            report = "Your administrative privileges cannot be verified. Failed to scrape."

        data = {
            "report": report
        }
        return render(request, 'scrape_page.html', {"data": data})

# get all filtered recipes

def get_all_filtered_recipes():
    recipe_list = []
    uid = m_user._getUser_Id_()
    word = recipes.get_recipe_name_to_find()
    all_recipes = db.child("recipe").get()
    if all_recipes.each() != None:
        for recipe in all_recipes.each():
            key = str(recipe.key())
            try:
                if recipe.val()["recipe_name"].lower().find(word.lower()) != -1:
                    _recipe_ = recipes.get_recipe(dict(recipe.val()), key, uid)
                    recipe_list.append(_recipe_)
            except KeyError:
                db.child("recipe").child(key).remove()
                continue
    return recipe_list

# Recipe Page
@csrf_exempt
def recipe_page(request):
    if recipes.get_scraped():
        recipes._get_all_recipes_()
        recipes.set_scraped(False)
    recipes.set_is_searched_for_recipes(False)
    recipes.set_fridge_recipes(None)
    recipes.set_recipe_name_to_find(None)
    recipes.set_filter_list(None)
    navigate_to_recipe_page = "/empty_my_fridge/recipe_list/"
    navigate_to_recipe_page += "?page=1"
    return HttpResponseRedirect(navigate_to_recipe_page)


def unique_filtered_recipes(recipes):
    unique_recipes = []
    for recipe in recipes:
        if recipe not in unique_recipes:
            unique_recipes.append(recipe)
    return unique_recipes


def set_active_filters(filters, removal, isCategory):
    new_filters = []
    new_unique_filters = []
    if isCategory:
        curr_filters = m_category.get_filter_list()
    else:
        curr_filters = recipes.get_filter_list()

    if curr_filters is not None:
        new_filters = curr_filters + filters
        for filter in new_filters:
            if filter not in new_unique_filters:
                new_unique_filters.append(filter)
    else:
        new_unique_filters = filters

    if len(removal) > 0:
        for filter in removal:
            if filter in new_unique_filters:
                new_unique_filters.remove(filter)
    if isCategory:
        m_category.set_filter_list(new_unique_filters)
        recipes.set_filter_list(None)
    else:
        recipes.set_filter_list(new_unique_filters)
        m_category.set_filter_list(None)


@csrf_exempt
def filter(recipe_list, filter_list, isComplete):
    filtered_recipes = []
    duplicate_recipes = []
    started = False

    if isComplete:
        for filter in filter_list:
            if len(filtered_recipes) == 0 and not started:
                started = True
                for recipe in recipe_list:
                    try:
                        categories = recipe["recipe_categories"]
                        if any(filter in f for f in categories):
                            filtered_recipes.append(recipe)
                    except KeyError:
                        pass
            else:
                for recipe in filtered_recipes:
                    try:
                        categories = recipe["recipe_categories"]
                        if not any(filter in f for f in categories):
                            duplicate_recipes.append(recipe)
                    except KeyError:
                        pass
    else:
        for filter in filter_list:
            for recipe in recipe_list:
                try:
                    categories = recipe["recipe_categories"]
                    if any(filter in f for f in categories):
                        filtered_recipes.append(recipe)
                except KeyError:
                    pass
    for duplicate in duplicate_recipes:
        if duplicate in filtered_recipes:
            filtered_recipes.remove(duplicate)
    filtered_recipes = unique_filtered_recipes(filtered_recipes)
    return filtered_recipes


@csrf_exempt
def recipe_list(request):
    found_results = False
    isSearch = False
    isComplete = False
    isRemoving = False
    isClearing = False
    isFilter = False
    to_remove = []
    f = request.GET.get("fridge")
    filters = request.POST.getlist('filter_data')
    remove_list = request.POST.getlist('remove_filter')
    clear_all = request.POST.get('clear')
    curr_filters = recipes.get_filter_list()
    m_category.set_sorting_type("name_A")
    recipes.set_fridge_sorting_type("name_A")

    if clear_all:
        isClearing = True

    if isClearing:
        recipes.set_filter_list(None)
    else:
        filter_style = request.POST.get('filter_style')
        if remove_list is not None:
            isRemoving = True
        if filter_style:
            isComplete = True

        if isRemoving:
            if curr_filters is not None:
                for option in remove_list:
                    to_remove.append(option)
        set_active_filters(filters, to_remove, False)

    all_recipes = []
    _recipe_name_ = None
    if recipes.get_is_searched_for_recipes():
        all_recipes = get_all_filtered_recipes()
        isSearch = True
        _recipe_name_ = recipes.get_recipe_name_to_find()
        # recipes.set_is_searched_for_recipes(False)
        if len(all_recipes) != 0:
            found_results = True
    else:
        all_recipes = recipes.get_all_recipes()
    scrollTop = 0
    keep_scroll_pos = False
    fridge_recipes = recipes.get_fridge_recipes()
    disp_fridge = (False, False)
    if request.method == "POST" or fridge_recipes:
        if ((sorting_type := request.POST.get('sorting'))
        != recipes.get_sorting_type() and sorting_type):
            recipes.set_sorting_type(sorting_type)
            recipes.set_fridge_recipes(fridge_recipes)
        elif request.method == "GET":
            pass
        else:
            recipes.set_fridge_recipes(None)
        fridge_recipes = recipes.get_fridge_recipes()
    
       
        if (request.POST.get('part')=="True" or fridge_recipes == "part"):
            recipes.set_fridge_recipes("part")
            if m_user._isNone_():
                activity_page = "/empty_my_fridge/login/?activity=recipe_list"
                return HttpResponseRedirect(activity_page)
            matches = Fridge_matches(all_recipes)
            uid = m_user._getUser_Id_()
            fridge_recipes = "part"
            all_recipes = []
            if(matches):
                all_recipes = matches["partial"]
                if all_recipes:
                    temp_recipes = []
                    for recipe in all_recipes:
                        try:
                            is_fav = recipe["stars"][uid] != None
                            recipe["user_saved"] = is_fav
                        except KeyError:
                            pass
                        temp_recipes.append(recipe)
                    all_recipes = temp_recipes
                    
            disp_fridge = (True, True)
        elif(fridge_lst:=request.POST.get('fridge')=="True" or fridge_recipes == "fridge"):
            recipes.set_fridge_recipes("fridge")
            if m_user._isNone_():
                activity_page = "/empty_my_fridge/login/?activity=recipe_list"
                return HttpResponseRedirect(activity_page)
            matches = Fridge_matches(all_recipes)
            all_recipes=[]
            fridge_recipes = "fridge"
            if(matches):
                all_recipes = matches["exact"]
                if all_recipes:
                    temp_recipes = []
                    for recipe in all_recipes:
                        try:
                            is_fav = recipe["stars"][uid] != None
                            recipe["user_saved"] = is_fav
                        except KeyError:
                            pass
                        temp_recipes.append(recipe)
                    all_recipes = temp_recipes
            disp_fridge = (True, False)
    if(fridge_recipes!="part" and recipes.get_sorting_type()[:3] == "mis"):
        recipes.set_sorting_type("name_A")

    all_recipes = sort_recipes(all_recipes, recipes.get_sorting_type()) or all_recipes
    
    if recipes.get_is_recipe_liked():
        scrollTop = recipes.get_recipe_list_position()
        recipes.set_is_recipe_liked(False)
        keep_scroll_pos = True
    filters = recipes.get_filter_list()
    if filters:
        isFilter = True
        all_recipes = filter(all_recipes, filters, isComplete)

    paginator = Paginator(all_recipes, 48)
    page = request.GET.get('page')
    recipes.set_recipes_current_page(page)

    try:
        curr_recipes = paginator.page(page)
    except PageNotAnInteger:
        curr_recipes = paginator.page(1)
    except EmptyPage:
        curr_recipes = paginator.page(paginator.num_pages)

    if m_user._isNone_():
        data = {
            "recipes": curr_recipes,
            "scrollTop": scrollTop,
            "found_results": found_results,
            "items": len(all_recipes),
            "isSearch": isSearch,
            "fridge": disp_fridge,
            "sorting_type": recipes.get_sorting_type(),
            "recipe_name": _recipe_name_,
            "active_filters": filters,
            "isFilter": isFilter,
            "fridge_recipes": fridge_recipes,

        }
        return render(request, 'recipes.html', {"data": data})
    else:
        if recipes.get_visited_pages(page) == -1:
            recipes.get_all_likes(m_user._getUser_Id_(), page)
            recipes.set_visited_pages(page)

        _user_ = m_user._getUser_()
        data = {
            "user": _user_,
            "recipes": curr_recipes,
            "scrollTop": scrollTop,
            "keep_scroll_pos": keep_scroll_pos,
            "found_results": found_results,
            "items": len(all_recipes),
            "isSearch": isSearch,
            "fridge": disp_fridge,
            "sorting_type": recipes.get_sorting_type(),
            "recipe_name": _recipe_name_,
            "active_filters": filters,
            "isFilter": isFilter,
            "fridge_recipes": fridge_recipes
        }
        return render(request, 'recipes.html', {"data": data})


@csrf_exempt
def category(request):
    cat = request.GET.get('category')
    isComplete = False
    isRemoving = False
    isClearing = False
    to_remove = []
    filters = request.POST.getlist('filter_data')
    remove_list = request.POST.getlist('remove_filter')
    clear_all = request.POST.get('clear')
    curr_filters = m_category.get_filter_list()
    recipes.set_sorting_type("name_A")
    recipes.set_fridge_sorting_type("name_A")

    if clear_all:
        isClearing = True

    if isClearing:
        m_category.set_filter_list(None)
    else:
        filter_style = request.POST.get('filter_style')
        if remove_list is not None:
            isRemoving = True
        if filter_style:
            isComplete = True

        if isRemoving:
            if curr_filters is not None:
                for option in remove_list:
                    to_remove.append(option)
        set_active_filters(filters, to_remove, True)

    m_category.set_category(cat)
    found_results = False
    recipe_lst = get_recipes_by_category(cat)
    if len(recipe_lst) != 0:
        found_results = True

    scrollTop = 0
    keep_scroll_pos = False
    if recipes.get_is_recipe_liked():
        scrollTop = recipes.get_recipe_list_position()
        recipes.set_is_recipe_liked(False)
        keep_scroll_pos = True
    filters = m_category.get_filter_list()
    if filters:
        recipe_lst = filter(recipe_lst, filters, isComplete)

    fridge_recipes = recipes.get_fridge_recipes()
    disp_fridge = (False, False)
    if request.method == "POST" or fridge_recipes:
        if(( sorting_type:=request.POST.get('sorting') )!= m_category.get_sorting_type() and sorting_type):
            m_category.set_sorting_type(sorting_type)
            recipes.set_fridge_recipes(fridge_recipes)
        elif request.method == "GET":
            pass
        else:
            recipes.set_fridge_recipes(None)
        fridge_recipes = recipes.get_fridge_recipes()
        if (request.POST.get('part')=="True" or fridge_recipes == "part"):
            recipes.set_fridge_recipes("part")
            if m_user._isNone_():
                activity_page = "/empty_my_fridge/login/?activity=recipe_list"
                return HttpResponseRedirect(activity_page)
            matches = Fridge_matches(recipe_lst)
            uid = m_user._getUser_Id_()
            fridge_recipes = "part"
            recipe_lst = []
            if(matches):
                recipe_lst = matches["partial"]
                if recipe_lst:
                    temp_recipes = []
                    for recipe in recipe_lst:
                        try:
                            is_fav = recipe["stars"][uid] != None
                            recipe["user_saved"] = is_fav
                        except KeyError:
                            pass
                        temp_recipes.append(recipe)
                    recipe_lst = temp_recipes
            disp_fridge = (True, True)
        elif(fridge_lst:=request.POST.get('fridge')=="True" or fridge_recipes=="fridge"):
            recipes.set_fridge_recipes("fridge")
            if m_user._isNone_():
                activity_page = "/empty_my_fridge/login/?activity=recipe_list"
                return HttpResponseRedirect(activity_page)
            uid = m_user._getUser_Id_()
            matches = Fridge_matches(recipe_lst)
            fridge_recipes = "fridge"
            recipe_lst = []
            if(matches):
                recipe_lst = matches["exact"]
                if recipe_lst:
                    temp_recipes = []
                    for recipe in recipe_lst:
                        try:
                            is_fav = recipe["stars"][uid] != None
                            recipe["user_saved"] = is_fav
                        except KeyError:
                            pass
                        temp_recipes.append(recipe)
                    recipe_lst = temp_recipes
            disp_fridge = (True, False)

    if(fridge_recipes!="part" and m_category.get_sorting_type()[:3] == "mis"):
        m_category.set_sorting_type("name_A")
    recipe_lst = sort_recipes(recipe_lst, m_category.get_sorting_type()) or recipe_lst

    paginator = Paginator(recipe_lst, 48)
    page = request.GET.get('page')
    if not page:
        page = "1"
    m_category.set_category_page(page)

    try:
        curr_recipes = paginator.page(page)
    except PageNotAnInteger:
        curr_recipes = paginator.page(1)
    except EmptyPage:
        curr_recipes = paginator.page(paginator.num_pages)

    user = m_user._getUser_()

    data = {
        "user": user,
        "activity": "categories",
        "recipe_lst": curr_recipes,
        "category": cat,
        "scrollTop": scrollTop,
        "keep_scroll_pos": keep_scroll_pos,
        "found_results": found_results,
        "items": len(recipe_lst),
        "fridge": disp_fridge,
        "sorting_type": m_category.get_sorting_type(),
        "active_filters": filters,
        "fridge_recipes" : fridge_recipes,
    }
    return render(request, 'category.html', {"data": data})


def get_recipes_by_category(category):
    all_recipes = db.child('recipe').get()
    uid = m_user._getUser_Id_()
    recipe_lst = []
    if all_recipes.each():
        for recipe in all_recipes.each():
            recipe_details = recipe.val()
            try:
                categories = recipe_details["recipe_categories"]
                if categories:
                    if any(category in c for c in categories):
                        key = str(recipe.key())
                        _recipe_ = recipes.get_recipe(
                            dict(recipe_details), key, uid)
                        recipe_lst.append(_recipe_)
            except KeyError:
                pass
    return recipe_lst


def get_recipes_by_ingredients(ingredient):
    all_recipes = db.child('recipe').get()
    recipe_lst = []
    uid = m_user._getUser_Id_()
    for recipe in all_recipes.each():
        recipe_details = recipe.val()
        try:
            ingredients = recipe_details["recipe_ingredients"]
            if ingredients:
                if any(ingredient in i for i in ingredients):
                    key = str(recipe.key())
                    _recipe_ = recipes.get_recipe(
                        dict(recipe_details), key, uid)
                    recipe_lst.append(_recipe_)
        except KeyError:
            pass
    return recipe_lst


def get_unique_recipes(categories, ingredients):
    uniques_recipes = []
    for recipe in categories:
        for _recipe_ in ingredients:
            if recipe == _recipe_:
                categories.remove(recipe)

    return categories + ingredients


@csrf_exempt
def get_recipes_by_category_ingredients(request):
    category_list = []
    ingred_list = []
    result_list = []
    value = None
    found_results = False
    recipes.set_sorting_type("name_A")
    recipes.set_fridge_sorting_type("name_A")
    if request.method == "GET":
        value = request.GET.get("category")
        m_category.set_category(value)
        category_list = get_recipes_by_category(value)
        ingred_list = get_recipes_by_ingredients(value)
        result_list = get_unique_recipes(category_list, ingred_list)
        if len(result_list) != 0:
            found_results = True

    scrollTop = 0
    keep_scroll_pos = False
    if recipes.get_is_recipe_liked():
        scrollTop = recipes.get_recipe_list_position()
        recipes.set_is_recipe_liked(False)
        keep_scroll_pos = True

    fridge_recipes = recipes.get_fridge_recipes()
    disp_fridge = (False, False)
    if request.method == "POST" or fridge_recipes:
        if(( sorting_type:=request.POST.get('sorting') )!= m_category.get_sorting_type() and sorting_type):
            m_category.set_sorting_type(sorting_type)
            recipes.set_fridge_recipes(fridge_recipes)
        elif request.method == "GET":
            pass
        else:
            recipes.set_fridge_recipes(None)
        fridge_recipes = recipes.get_fridge_recipes()
        if (request.POST.get('part')=="True" or fridge_recipes == "part"):
            recipes.set_fridge_recipes("part")
            if m_user._isNone_():
                activity_page = "/empty_my_fridge/login/?activity=recipe_list"
                return HttpResponseRedirect(activity_page)
            matches = Fridge_matches(result_list)
            fridge_recipes = "part"
            result_list = []
            uid = m_user._getUser_Id_()
            if(matches):
                result_list = matches["partial"]
                if result_list:
                    temp_recipes = []
                    for recipe in result_list:
                        try:
                            is_fav = recipe["stars"][uid] != None
                            recipe["user_saved"] = is_fav
                        except KeyError:
                            pass
                        temp_recipes.append(recipe)
                    result_list = temp_recipes
            disp_fridge = (True, True)
        elif(fridge_lst:=request.POST.get('fridge')=="True"or fridge_recipes=="fridge"):
            recipes.set_fridge_recipes("fridge")
            if m_user._isNone_():
                activity_page = "/empty_my_fridge/login/?activity=recipe_list"
                return HttpResponseRedirect(activity_page)
            matches = Fridge_matches(result_list)
            result_list = []
            uid = m_user._getUser_Id_()
            if(matches):
                result_list = matches["exact"]
                if result_list:
                    temp_recipes = []
                    for recipe in result_list:
                        try:
                            is_fav = recipe["stars"][uid] != None
                            recipe["user_saved"] = is_fav
                        except KeyError:
                            pass
                        temp_recipes.append(recipe)
                    result_list = temp_recipes
            disp_fridge = (True, False)

    if(fridge_recipes!="part" and m_category.get_sorting_type()[:3] == "mis"):
        m_category.set_sorting_type("name_A")
    result_list = sort_recipes(result_list, m_category.get_sorting_type()) or result_list
    result_list = unique_filtered_recipes(result_list)
    paginator = Paginator(result_list, 48)
    page = request.GET.get('page')
    if not page:
        page = "1"
    m_category.set_category_page(page)

    try:
        curr_recipes = paginator.page(page)
    except PageNotAnInteger:
        curr_recipes = paginator.page(1)
    except EmptyPage:
        curr_recipes = paginator.page(paginator.num_pages)

    user = m_user._getUser_()
    data = {
        "user": user,
        "activity": "search",
        "recipe_lst": curr_recipes,
        "category": value,
        "scrollTop": scrollTop,
        "keep_scroll_pos": keep_scroll_pos,
        "found_results": found_results,
        "items": len(result_list),
        "fridge": disp_fridge,
        "sorting_type": m_category.get_sorting_type(),
    }

    return render(request, 'category.html', {"data": data})


# Search Page
@csrf_exempt
def search(request):
    if request.method == "GET":
        recipe_name_to_find = request.GET.get("recipe_to_filter")
        if len(recipe_name_to_find) > 0:
            recipes.set_recipe_name_to_find(recipe_name_to_find)
            recipes.set_is_searched_for_recipes(True)
            recipes.set_filter_list(None)
        navigate_to_recipe_page = "/empty_my_fridge/recipe_list/"
        if recipes.get_recipes_current_page():
            navigate_to_recipe_page += "?page=" + recipes.get_recipes_current_page()
        else:
            navigate_to_recipe_page += "?page=1"
    return HttpResponseRedirect(navigate_to_recipe_page)


# Actions (Recipe onClick)
@csrf_exempt
def fav_recipe_onClick(request):
    if m_user._isNone_():
        activity_page = None
        activity_page = request.POST.get("activity")
        if activity_page:
            activity_page = "/empty_my_fridge/login/?activity={0}".format(
                activity_page)
        else:
            activity_page = "/empty_my_fridge/login/"

        return HttpResponseRedirect(activity_page)
    else:
        if request.method == "POST":
            uid = m_user._getUser_Id_()
            recipe_id = request.POST.get("recipe_id")
            navigate = request.POST.get("navigate")
            scrollTop = request.POST.get("scroll_y")
            isSearch = request.POST.get("isSearch")
            isFridge_recipes = request.POST.get("fridge_recipes")
            if isFridge_recipes:
                recipes.set_fridge_recipes(isFridge_recipes)

            if isSearch == "True":
                recipes.set_is_searched_for_recipes(True)
            recipes.set_recipe_list_position(scrollTop)
            recipes.set_is_recipe_liked(True)
            today = date.today()
            time_now = time.time()
            time_liked = {
                "time": time_now,
            }
            _recipe_data_ = db.child("user_fav_recipes").child(
                uid).child(recipe_id).get().val()
            if _recipe_data_ != None:
                recipes.set_recipe_unLiked(recipe_id)
                db.child("user_fav_recipes").child(
                    uid).child(recipe_id).remove()
                db.child("recipe").child(recipe_id).child(
                    "stars").child(uid).remove()
            else:
                recipes.set_recipe_liked(recipe_id)
                db.child("user_fav_recipes").child(
                    uid).child(recipe_id).set(time_liked)
                db.child("recipe").child(recipe_id).child(
                    "stars").child(uid).set(time_liked)
            if navigate == "/empty_my_fridge/recipe_list/":
                navigate += "?page=" + recipes.get_recipes_current_page()
            elif navigate == "/empty_my_fridge/categories/" or navigate == "/empty_my_fridge/search/":
                navigate += "?category=" + m_category.get_category() + "&page=" + m_category.get_category_page()
            elif navigate == "/empty_my_fridge/public_fav_recipes/":
                navigate += "?id=" + m_user.get_friend_id()
            elif navigate == "/empty_my_fridge/fridge/recipes":
                navigate += "?page=" + m_category.get_category_page()

            return HttpResponseRedirect(navigate)


@csrf_exempt
def upload_image(request):
    try:
        file = request.FILES['img']
        uid = m_user._getUser_Id_()
        if uid:
            _dir_ = "images/{0}/{1}".format(uid, file.name)
            img_details = fb_storage.child(_dir_).put(
                file, request.session['token_id'])
            link = fb_storage.child(_dir_).get_url(
                img_details["downloadTokens"])

            userData = {
                'image': link,
            }
            db.child("users").child(uid).update(userData)
            _user_ = dict(db.child("users").child(uid).get().val())
            m_user._setUser_(_user_)
    except Exception:
        pass

    return HttpResponseRedirect('/empty_my_fridge/edit_profile/')


##Authentication (Login and Register)
@csrf_exempt
def login(request):
    activity_page = None
    cat = None
    if request.method == 'GET':
        activity_page = request.GET.get("activity")
        cat = request.GET.get("category")
        id = request.GET.get("id")
        if not cat:
            cat = m_category.get_category()
    if not activity_page:
        activity_page = m_activity.get_activity_page()
        if not activity_page:
            activity_page = "home"

    data = {
        "activity": activity_page,
        "category": cat,
        "id" : id,
    }
    m_activity.set_activity_page(None)
    return render(request, 'login.html', {"data": data})


@csrf_exempt
def _login_(request):
    activity_page = None
    if request.method == 'GET':
        email = request.GET.get('email')
        password = request.GET.get('pass')
        activity_page = request.GET.get("activity")
        cat = request.GET.get("category")
        id = request.GET.get("id")
        if activity_page == "recipe_list":
            page = recipes.get_recipes_current_page()
            activity_page = "/empty_my_fridge/{0}/?page={1}".format(
                activity_page, page)
        elif (activity_page == "categories" or activity_page == "search") and cat != "None":
            page = m_category.get_category_page()
            activity_page = "/empty_my_fridge/{0}/?category={1}&page={2}".format(
                activity_page, cat, page)
        elif activity_page == "recipe_view_page":
            activity_page = "/empty_my_fridge/{0}/?id={1}".format(activity_page, id)
        elif not activity_page:
            activity_page = "/empty_my_fridge/home/"
        else:
            activity_page = "/empty_my_fridge/{0}/".format(activity_page)

        try:
            user = auth_fb.sign_in_with_email_and_password(email, password)
            if user != None:
                user = auth_fb.refresh(user['refreshToken'])
                uid = user["userId"]
                user_info = auth_fb.get_account_info(user['idToken'])
                is_verified = user_info["users"][0]["emailVerified"]
                if is_verified:
                    _user_ = db.child("users").child(uid).get().val()
                    m_user._setUser_Id_(uid)
                    user_details = dict(_user_)
                    m_user._setUser_(user_details)
                    request.session['token_id'] = user['idToken']
                    request.session['uid'] = uid
                else:
                    try:
                        del request.session['token_id']
                        del request.session["uid"]
                    except KeyError:
                        pass
                    auth_fb.send_email_verification(user['idToken'])
                    auth.logout(request)
                    msg = error_message('NOT_VERIFIED')
                    data = {
                        "message": msg
                    }

                    return render(request, "login.html", {"data": data})

        except Exception as e:
            # logging.exception('')
            response = e.args[0].response
            error = response.json()['error']
            print(error['message'])
            msg = error_message(error['message'])
            data = {
                "message": msg
            }
            return render(request, "login.html", {"data": data})
    return HttpResponseRedirect(activity_page)


@csrf_exempt
def register(request):
    return render(request, 'register.html')


@csrf_exempt
def _register_(request):
    if request.method == 'POST':
        email = request.POST.get('email')
        password = request.POST.get('pass')
        name = request.POST.get('name')
        data = {
            "message": "A verification link was sent to your email. Please, follow the link to verify your email.",
            "msg_type": "success"
        }
        try:
            user = auth_fb.create_user_with_email_and_password(email, password)
            user['displayName'] = name
            uid = user['localId']
            email = user['email']
            index_of_at = email.find("@")
            username = email[:index_of_at] + uid[:5]
            today = date.today()
            joined = today.strftime("%B %d, %Y")

            userData = {
                'name': name,
                'email': email,
                'joined': joined,
                'userID': uid,
                'username': username.lower(),
                'image': 'https://react.semantic-ui.com/images/wireframe/square-image.png'
            }
            db.child('users').child(uid).set(userData)
            auth_fb.send_email_verification(user["idToken"])

        except Exception as e:
            # logging.exception('')
            response = e.args[0].response
            error = response.json()['error']
            msg = error_message(error['message'])
            data["message"] = msg
            data["msg_type"] = "error"

        return render(request, "register.html", {"data": data})

# Profile Page (Profile--about me, Edit Profile and Save new profile information, Account Settings, Recover Password, User Favorite Recipes)

# Profile Page (Profile--about me, Edit Profile and Save new profile information, Account Settings, Recover Password, User Favorite Recipes)


@csrf_exempt
def profile(request):
    if m_user._isNone_():
        m_activity.set_activity_page("profile")
        return HttpResponseRedirect("/empty_my_fridge/login/")
    else:
        uid = m_user._getUser_Id_()
        user = db.child("users").child(uid).get().val()
        user_details = dict(user)
        m_user._setUser_(user_details)
        data = {
            "user": user_details
        }
        return render(request, "profile.html", {"data": data})

@csrf_exempt
def public_profile(request):
    user_details = {}
    pUser_details = {}
    if m_user._isNone_():
        return HttpResponseRedirect("/empty_my_fridge/login/")
    else:
        #set the user that's still logged in
        uid = m_user._getUser_Id_()
        user = db.child("users").child(uid).get().val()
        user_details = dict(user)
        m_user._setUser_(user_details)
        #if a user was posted (which is the only way to get to this page),
        #   then, grab that user's ID and return an object with that user's info
        friend_id = request.GET.get("id")
        m_user.set_friend_id(friend_id)
        if friend_id:
            pUser = db.child("users").child(friend_id).get().val()
            pUser_details = dict(pUser)
            #pUser._setUser_(user_details)
    data = {
        "user": user_details,
        "pUser": pUser_details
    }
    return render(request, "public_profile.html", {"data": data})

@csrf_exempt
def recipe_view_page(request):
    user = m_user._getUser_()
    uid = m_user._getUser_Id_()
    print_list = []
    recipe_id = request.GET.get("id")
    link = None
    this_recipe = None
    if request.method == "POST":
        try:
            file = request.FILES['img']
            _dir_ = "recipe_images/{0}/{1}".format(uid, file.name)
            img_details = fb_storage.child(_dir_).put(file, request.session['token_id'])
            link = fb_storage.child(_dir_).get_url(img_details["downloadTokens"])
        except Exception:
            pass
    if recipe_id:
        recipe_image_update = {
            "recipe_image": link
        }
        this_recipe = db.child("recipe").child(recipe_id).get().val()
        if this_recipe and link:
            db.child("recipe").child(recipe_id).update(recipe_image_update)
            db.child("users").child(uid).child("recipes").child(recipe_id).update(recipe_image_update)
            this_recipe["recipe_image"] = link

        elif not this_recipe and link:
        
            db.child("users").child(uid).child("recipes").child(recipe_id).update(recipe_image_update)

        if not this_recipe:
            this_recipe = db.child("users").child(uid).child("recipes").child(recipe_id).get().val()

        if not this_recipe:
            friend_id = m_user.get_friend_id()
            this_recipe = db.child("users").child(friend_id).child("recipes").child(recipe_id).get().val()

        try:
            if uid:
                if this_recipe["uid"] == uid:
                    this_recipe["belongsToUser"] = True
        except KeyError:
            this_recipe["belongsToUser"] = False
            pass
        for (measurement, ingredient) in zip(this_recipe["measurements"], this_recipe["recipe_ingredients"]):
            combo = measurement + ": " + ingredient
            print_list.append(combo)
        
    
    data = {
        "recipe": this_recipe,
        "ingredients": print_list,
        "recipe_id": recipe_id,
        "user" : user,
    }
    return render(request, 'recipe_view_page.html', {"data": data})


@csrf_exempt
def user_friends(request):
    remove = False
    if m_user._isNone_():
        return HttpResponseRedirect("/empty_my_fridge/login/")
    else:
        user = m_user._getUser_()
        uid = m_user._getUser_Id_()
        friends_list = []
        friends = db.child("users").child(uid).child("friends").get().val()
        num_of_friends = 0
        if friends and friends.items() != None:
            for _key_,value in friends.items():
                #_key_ = str(fren.key()) #grabs friend's unique id
                thisFren = db.child("users").child(_key_).get().val() #what does .val() do at the end?
                #nt(value["added_back"])
                if value["added_back"]:
                    friend_details = dict(thisFren)
                    friends_list.append(friend_details)

        if request.method == "POST":
            print("user is being DELETED...")
            deny = request.POST.get("deny")
            thisFren = db.child("users").child(deny).get().val() #what does .val() do at the end?
            #once you find the right person...
            if thisFren:
                #delete them from my friend list...
                db.child("users").child(uid).child("friends").child(deny).remove()
                remove = True
                return HttpResponseRedirect("/empty_my_fridge/friends/")
        
        num_of_friends = len(friends_list)
        data = {
            "user": user,
            "friends": friends_list,
            "num_of_friends": num_of_friends
        }

        return render(request, 'friends.html', {"data": data})

@csrf_exempt
def public_friends(request):
    remove = False
    if m_user._isNone_():
        return HttpResponseRedirect("/empty_my_fridge/login/")
    else:
        user = m_user._getUser_()
        uid = m_user._getUser_Id_()
        #get friend's object
        friend_id = request.GET.get("id")

        if friend_id:
            pUser = db.child("users").child(friend_id).get().val()
            pUser_details = dict(pUser)

        friends_list = []
        friends = db.child("users").child(friend_id).child("friends").get().val()
        num_of_friends = 0
        if friends.items() != None:
            for _key_,value in friends.items():
                if value["added_back"]:
                    thisFren = db.child("users").child(_key_).get().val()
                    friend_details = dict(thisFren)
                    friends_list.append(friend_details)
        num_of_friends = len(friends_list)
        data = {
            "user": user,
            "pUser": pUser,
            "friends": friends_list,
            "num_of_friends": num_of_friends
        }

        return render(request, 'public_friends.html', {"data": data})

@csrf_exempt
def public_fav_recipes(request):
    if m_user._isNone_():
            return HttpResponseRedirect("/empty_my_fridge/login/")
    else:
        user = m_user._getUser_()
        uid = m_user._getUser_Id_()
        friend_id = m_user.get_friend_id()
        fav_recipes_list = []
        num_of_fav_recipes = 0
        pUser = None
        if friend_id:
            pUser = db.child("users").child(friend_id).get().val()
            fav_recipes = db.child("user_fav_recipes").child(friend_id).get()
            if fav_recipes.each():
                for recipe in fav_recipes.each():
                    _key_ = str(recipe.key())
                    user_fav_recipe = db.child("recipe").child(_key_).get().val()
                    if user_fav_recipe:
                        recipe_details = dict(user_fav_recipe)
                        recipe_details = recipes.get_recipe(recipe_details, _key_, uid)
                        fav_recipes_list.append(recipe_details)

            num_of_fav_recipes = len(fav_recipes_list)
        data = {
            "user": user,
            "pUser": pUser,
            "fav_recipes": fav_recipes_list,
            "num_of_fav_recipes": num_of_fav_recipes,
        }

        return render(request, 'public_fav_recipes.html', {"data": data})

@csrf_exempt
def public_personal_recipes(request):
    recipe_details = None
    msg = None
    msg_type = None
    userRecipe = None
    no_rec = None
    friend_recipe_list = []

    if m_user._isNone_():
        return HttpResponseRedirect("/empty_my_fridge/login/") 
    else:
        user = m_user._getUser_()
        uid = m_user._getUser_Id_()
        friend_id = m_user.get_friend_id()
        pUser = None
        if friend_id:
            pUser = db.child("users").child(friend_id).get().val()

            my_recipes = db.child("users").child(friend_id).child("recipes").get().each()
            if my_recipes:
                for recipe in my_recipes:
                    recipe_details = recipe.val()
                    if recipe_details["privacy"] != "0":
                        friend_recipe_list.append(recipe_details)   
            else:
                no_rec = "Looks like they don't have any personal recipes at the moment..."
    data = {
        "user": user,
        "pUser": pUser,
        "message": msg,
        "msg_type": msg_type,
        "my_recipes": friend_recipe_list,
        "no_rec": no_rec,
    }
    return render(request, 'public_personal_recipes.html', {"data": data, })


def remove_white_spaces(items):
    new_items = []
    for item in items:
        new_items.append(item.strip().lower())
    return new_items


@csrf_exempt
def friend_requests(request):
    print("Friend Request!")
    if m_user._isNone_():
        return HttpResponseRedirect("/empty_my_fridge/login/")
    else:
        user = m_user._getUser_()
        uid = m_user._getUser_Id_()
        users = db.child("users").get()
        found = False
        isSent = False
        isSub = False
        thisUser = None
        friend_added = False
        friend_deleted = False

        friends_list = []
        friends = db.child("users").child(uid).child("friends").get().val()
        num_of_friends = 0
    

        #we're posting an email or id to check
        if request.method == "POST":
            friends = db.child("users").child(uid).child("friends").get().val()
            
            isSent = True
            email = request.POST.get("email")
            username = request.POST.get("username")
            accept = request.POST.get("accept")
            deny = request.POST.get("deny")
            #they chose to use email....
            if email:
                for person in users.each():
                    person = dict(person.val())
                    if person["email"] == email:
                        thisUser = person
                        break
            elif username:
                for person in users.each():
                    person = dict(person.val())
                    if person["username"] == username:
                        thisUser = person
                        break
            if accept:
                isSub = True
                print("user is being ADDED.... ")
                #so if you actually have friends...
                #if friends:
                    #look through them...
                   # for fren in friends.each():
                thisFren = db.child("users").child(uid).child("friends").child(accept).get().val() #what does .val() do at the end?
                #set them true on my friend list...
                db.child("users").child(uid).child("friends").child(accept).update({"added_back": "True"})
               # thisFren["added_back"] = True
                
                #and add me as a friend on theirs...
                me = dict(user)
                me = {
                    "added_back": True,
                    "userID": uid
                }
                db.child("users").child(accept).child("friends").child(uid).set(me)
                friend_added = True
            if deny:
                isSub = True
                print("user is being DELETED...")
                thisFren = db.child("users").child(deny).get().val() #what does .val() do at the end?
                #once you find the right person...
                if thisFren:
                    #delete them from my friend list...
                    db.child("users").child(uid).child("friends").child(deny).remove()
                    friend_deleted = True

            if thisUser:
                #thisEmail = thisUser["email"]
                #once we've found the right user, add us to their friends list!
                _key_ = thisUser["userID"] #grabs person's unique id
                thisFren = db.child("users").child(_key_).get().val()
                if thisFren:
                    me = {
                        "added_back": False,
                        "userID": uid
                    }
                    # me["added_back"] = False
                    # me["user_id"] = uid
                    # their_friends.append(me)
                    
                    db.child("users").child(_key_).child("friends").child(uid).set(me)
                    found = True
            if accept or deny:
                return HttpResponseRedirect("/empty_my_fridge/friend_requests/")
            #they chose to use userID...
    
        if friends:
            for key, value in friends.items():
                thisFren = db.child("users").child(key).get().val()
 
                #once you find the right person...
                if not value["added_back"]:
                    friends_list.append(dict(thisFren))
        num_of_friends = len(friends_list)
        data = {
            "user": user,
            "found": found,
            "isSent": isSent,
            "isSub": isSub,
            "friends": friends_list,
            "num_of_friends": num_of_friends,
            "friend_added": friend_added,
            "friend_deleted": friend_deleted
        }
    
    return render(request, 'friend_requests.html', {"data": data})

@csrf_exempt
def add_friend(request):
    if m_user._isNone_():
        return HttpResponseRedirect("/empty_my_fridge/login/")
    else:
        user = m_user._getUser_()
        uid = m_user._getUser_Id_()
        friends_list = []
        friends = db.child("users").child(uid).child("friends").get()
        if request.method == "POST":
            frenID = request.POST.get("frenID")
            if friends.each() != None:
                for fren in friends.each():
                    _key_ = str(fren.key()) #grabs friend's unique id
                    thisFren = db.child("users").child(_key_).get().val() #what does .val() do at the end?
                    #once you find the right person...
                    if thisFren["userID"] == frenID:
                        #add them to my friend list...
                        friends_list.remove(fren)
                        their_friends = db.child("users").child(uid).child("friends").get()
                        
                        #and add me to theirs...
                        me = dict(user)
                        me["added_back"] = True
                        me["user_id"] = uid
                        their_friends.append(me)
                        friend_added = True


        
        data = {
            "user": user,
            "friend_added": friend_added
        }

        return render(request, 'friends.html', {"data": data})

@csrf_exempt
def delete_friend(request):
    if m_user._isNone_():
        return HttpResponseRedirect("/empty_my_fridge/login/")
    else:
        user = m_user._getUser_()
        uid = m_user._getUser_Id_()
        friends_list = []
        friends = db.child("users").child(uid).child("friends").get()
        friend_deleted = False
        if request.method == "POST":
            frenID = request.POST.get("frenID")
            if friends.each() != None:
                for fren in friends.each():
                    _key_ = str(fren.key()) #grabs friend's unique id
                    thisFren = db.child("users").child(_key_).get().val() #what does .val() do at the end?
                    #once you find the right person...
                    if thisFren["userID"] == frenID:
                        #remove them from my friend list...
                        friends_list.remove(fren)
                        their_friends = db.child("users").child(uid).child("friends").get()
                        
                        #and remove me from theirs...
                        me = dict(user)
                        me["added_back"] = True
                        me["user_id"] = uid
                        their_friends.remove(me)
                        friend_added = True

        
        data = {
            "user": user,
            "friend_added": friend_added
        }

        return render(request, 'friends.html', {"data": data})


@csrf_exempt
def edit_profile(request):
    if m_user._isNone_():
        m_activity.set_activity_page("edit_profile")
        return HttpResponseRedirect("/empty_my_fridge/login/")
    else:
        user = m_user._getUser_()
        data = {
            'user': user,
        }
        return render(request, 'edit_profile.html', {"data": data})


@csrf_exempt
def personal_recipes(request):
    recipe_details = None
    msg = None
    msg_type = None
    userRecipe = None
    no_rec = None
    my_recipes = []
    my_recipe_list = []
    link = "https://firebasestorage.googleapis.com/v0/b/empty-my-fridge-ff73c.appspot.com/o/cutePlate.jpg?alt=media&token=229f016f-e151-4bf7-b569-d6a07a6a2c18"

    if m_user._isNone_():
        return HttpResponseRedirect("/empty_my_fridge/login/") 
    else:
        user = m_user._getUser_()
        uid = m_user._getUser_Id_()
        my_recipes = db.child("users").child(uid).child("recipes").get().val()
        if not my_recipes:
            print("NO RECIPES")
            no_rec = "There are no recipes to show... use the form on the left to add some!"
        
        if request.method == "POST":

            title = request.POST.get("title")
            if not title:
                recipe_id = request.POST.get("delete_recipe")
                db.child("users").child(uid).child("recipes").child(recipe_id).remove()
                db.child("recipe").child(recipe_id).remove()
            else:
                ingredients = request.POST.get("ingredients").split(",")
                ingredients = remove_white_spaces(ingredients)
                measurements = request.POST.get("measurements").split(",")
                measurements = remove_white_spaces(measurements)
                
                if len(ingredients) == len(measurements):
                    r_key = db.generate_key()
                    description = request.POST.get("description")
                    recipe_categories = request.POST.get("recipe_categories").split(",")
                    recipe_categories = remove_white_spaces(recipe_categories)
                    steps = request.POST.get("steps").splitlines()
                    privacy = request.POST.get("privacy")
                    clicked = request.POST.get("rec_link")

                    userRecipe = {
                    'recipe_name': title,
                    'recipe_id': r_key,
                    'description': description,
                    'recipe_categories': recipe_categories,
                    'steps': steps,
                    'recipe_ingredients': ingredients,
                    'measurements': measurements,
                    'privacy': privacy,
                    'recipe_image': link,
                    'uid': uid
                    }
                    if privacy == "2":
                        thisIng = db.child("all_ingredients").get().val()
                        if thisIng:
                            thisIng = list(dict.fromkeys(thisIng+ingredients))
                        else:
                            thisIng = ingredients
                        
                        db.child("all_ingredients").set(thisIng)
                        db.child("recipe").child(r_key).set(userRecipe)
                    
                    db.child("users").child(uid).child("recipes").child(r_key).set(userRecipe)
                    
                    msg = "Your recipe has been created!"
                    msg_type = "success"
                else:
                    msg = "Looks like you're missing an ingredient or a measurement... try again!"
                    msg_type = "error"

    my_recipes = db.child("users").child(uid).child("recipes").get().each()

    if my_recipes:
        for recipe in my_recipes:
            recipe_details = recipe.val()
            key = str(recipe.key())
            recipe_details["recipe_id"] = key
            my_recipe_list.append(recipe_details)
    
    data = {
        "user": user,
        "message": msg,
        "msg_type": msg_type,
    }
    return render(request, 'personal_recipes.html', {"data": data, "my_recipes": my_recipe_list, "no_rec": no_rec})


@csrf_exempt
def save_profile(request):
    user_details = m_user._getUser_()
    uid = m_user._getUser_Id_()
    msg = error_message("err")
    msg_type = "error"
    if m_user._isNone_():
        return HttpResponseRedirect("/empty_my_fridge/login/")
    else:
        if request.method == "POST":
            name = request.POST.get("name")
            bio = request.POST.get("bio")
            country = request.POST.get("country")
            blog = request.POST.get("blog")
            state = request.POST.get("state")
            userData = {
                'name': name,
                'bio': bio,
                'country': country,
                'blog': blog,
                'state': state
            }
            try:
                db.child("users").child(uid).update(userData)
                _user_ = dict(db.child("users").child(uid).get().val())
                m_user._setUser_(_user_)
                user_details = _user_
                msg = "Changes saved successfully."
                msg_type = "success"
            except Exception as e:
                pass
    data = {
        'user': user_details,
        "message": msg,
        "msg_type": msg_type
    }

    return render(request, 'edit_profile.html', {"data": data})


@csrf_exempt
def account_settings(request):
    if m_user._isNone_():
        return HttpResponseRedirect("/empty_my_fridge/login/")
    else:
        user_details = m_user._getUser_()
        uid = m_user._getUser_Id_()
        isAdmin = False
        admins = db.child("admin").child(
            "UPLwshBH98OmbVivV").child("scrapers").get().val()
        for admin in admins:
            if str(admin) == str(uid):
                isAdmin = True
                break
        data = {
            'user': user_details,
            'admin': isAdmin,
            "msg_type": m_message.get_msg_type(),
            "message": m_message.get_message()
        }
        m_message.set_message(None)
        m_message.set_msg_type(None)
        return render(request, 'account_settings.html', {"data": data})


@csrf_exempt
def reset_password(request):
    data = {
        "msg_type": m_message.get_msg_type(),
        "message": m_message.get_message()
    }
    m_message.set_message(None)
    m_message.set_msg_type(None)
    return render(request, 'reset_password_page.html', {"data": data})


@csrf_exempt
def recover_password(request):
    msg = error_message("err")
    msg_type = "error"
    email = request.POST.get("email")
    activity = request.POST.get("activity")
    try:
        auth_fb.send_password_reset_email(email)
        msg = "A password recovery link has been sent to your email."
        msg_type = "success"
    except Exception as e:
        response = e.args[0].response
        error = response.json()['error']
        msg = error_message(error['message'])

    activity = '/empty_my_fridge/{0}/'.format(activity)
    m_message.set_message(msg)
    m_message.set_msg_type(msg_type)

    return HttpResponseRedirect(activity)


@csrf_exempt
def user_fav_recipes(request):
    if m_user._isNone_():
        return HttpResponseRedirect("/empty_my_fridge/login/")
    else:
        user = m_user._getUser_()
        uid = m_user._getUser_Id_()
        fav_recipes_list = []
        fav_recipes = db.child("user_fav_recipes").child(uid).get()
        num_of_fav_recipes = 0
        if fav_recipes.each() != None:
            for recipe in fav_recipes.each():
                _key_ = str(recipe.key())
                user_fav_recipe = db.child("recipe").child(_key_).get().val()
                if user_fav_recipe:
                    recipe_details = dict(user_fav_recipe)
                    recipe_details["user_saved"] = True
                    recipe_details["recipe_id"] = _key_
                    fav_recipes_list.append(recipe_details)

        num_of_fav_recipes = len(fav_recipes_list)
        data = {
            "user": user,
            "fav_recipes": fav_recipes_list,
            "num_of_fav_recipes": num_of_fav_recipes,
        }

        return render(request, 'user_fav_recipes.html', {"data": data})


# User Error Messages Display
def error_message(type):

    if type.find("WEAK_PASSWORD") != -1:
        type = "WEAK_PASSWORD"

    if type.find("TOO_MANY_ATTEMPTS_TRY_LATER") != -1:
        type = "TOO_MANY_ATTEMPTS_TRY_LATER"

    return {
        "EMAIL_EXISTS": "This email is already in use. Try a different email!",
        "WEAK_PASSWORD": "Password should be at least 6 characters.",
        "INVALID_EMAIL": "Either your email or password is incorrect. Try again.",
        "INVALID_PASSWORD": "Either your email or password is incorrect. Try again.",
        "EMAIL_NOT_FOUND": "This email does not exist anywhere on our services.",
        "WRONG_EMAIL": "Please make sure you are using a valid email address.",
        "NOT_VERIFIED": "Your email is not verified! Please, follow the link in your email to verify your account.",
        "TOO_MANY_ATTEMPTS_TRY_LATER": "There have been too many unsuccessful login attempts. Please try again later."

    }.get(type, "An unknown error has occurred")

# logOut


@csrf_exempt
def _logout_(request):
    m_user._setUser_(None)
    m_user._setUser_Id_(None)
    #del m_user
    try:
        del request.session['token_id']
        del request.session["uid"]
    except KeyError:
        pass
    auth.logout(request)
    return HttpResponseRedirect("/empty_my_fridge/home/")


@csrf_exempt
def fridge(request):
    uid = None
    if m_user._isNone_():
        activity_page = "/empty_my_fridge/login/?activity=fridge"
        return HttpResponseRedirect(activity_page)
    else:
        uid = m_user._getUser_Id_()
        #user = m_user._getUser_()

    all_ingredients = []
    _all_ingredients_ = db.child("all_ingredients").get().val()
    if(_all_ingredients_):
        for ingredient in _all_ingredients_:
            if ingredient:
                all_ingredients.append(ingredient)
    
    if len(all_ingredients) > 0:
        all_ingredients = sorted(all_ingredients)
    fridge_ingredients = db.child("users").child(uid).child("Fridge").get().val()  # database is cleared of null values
    if fridge_ingredients:
        fridge_ingredients = sorted(fridge_ingredients)

    search_aing = request.GET.get('search_all_ingredients')
    search_fing = request.GET.get('search_personal_ingredients')

    chk_food = request.POST.getlist('sav_ing')
    del_food = request.POST.getlist('del_ing')
    popup={}
    

    if(request.POST.get('remove_all')):
        db.child("users").child(uid).child("Fridge").remove()
        popup={"all":["Everything"]}

    if del_food:
        popup={"del":del_food}
        for food in del_food:
            db.child("users").child(uid).child("Fridge").child(food).remove()

    if search_aing:
        all_ingredients = [i for i in all_ingredients if search_aing in i]
        if not all_ingredients:
            all_ingredients = []
    if search_fing:
        fridge_ingredients = [i for i in fridge_ingredients if search_fing in i]
        if not fridge_ingredients:
            fridge_ingredients = []
   
    if chk_food and uid:
        new_ingredients = {}
        if fridge_ingredients:
            disj = list(set(chk_food)-set(fridge_ingredients))
            popup={"add":disj}
            for x in disj:
                new_ingredients[x] = x
            db.child("users").child(uid).child(
                "Fridge").update(new_ingredients)

        else:
            for x in chk_food:
                new_ingredients[x] = x
            db.child("users").child(uid).child("Fridge").set(new_ingredients)

    if fridge_ingredients:
        fing_len = len(fridge_ingredients)
    else:
        fing_len = 0

    btnclick = request.POST.get('find_Recipe')
    if btnclick:
        return HttpResponseRedirect("/empty_my_fridge/fridge/recipes")
    
    
    context = {
        "ingredients" : all_ingredients, 
        'fing' : fridge_ingredients,
        "user": m_user._getUser_(),
        'fing_amount' : fing_len,
        "popup":popup
        }
    return render(request, 'fridge.html', context )

@csrf_exempt
def fridge_recipes(request):
    if m_user._isNone_():
        activity_page = "/empty_my_fridge/login/?activity=fridge"
        return HttpResponseRedirect(activity_page)
    recipes.set_sorting_type("name_A")
    m_category.set_sorting_type("name_A")
    all_recipes = recipes.get_all_recipes()
    matches = Fridge_matches(all_recipes)
    uid = m_user._getUser_Id_()
    possible_recipes = matches["exact"]
    temp_recipes = []
    if possible_recipes:
        for recipe in possible_recipes:
            try:
                is_fav = recipe["stars"][uid] != None
                recipe["user_saved"] = is_fav
            except KeyError:
                pass
            temp_recipes.append(recipe)
        possible_recipes = temp_recipes
    disp_partial = False
    fridge_recipes = recipes.get_fridge_recipes()
    if request.method == "POST" or fridge_recipes:
        if(( sorting_type:=request.POST.get('sorting') )!= recipes.get_fridge_sorting_type() and sorting_type):
            recipes.set_fridge_sorting_type(sorting_type)
            recipes.set_fridge_recipes(fridge_recipes)
        elif request.method == "GET":
            pass
        else:
            recipes.set_fridge_recipes(None)
        
        fridge_recipes = recipes.get_fridge_recipes()
        if (request.POST.get('part') == "True" or fridge_recipes == "part"):
            recipes.set_fridge_recipes("part")
            fridge_recipes = "part"
            disp_partial = True
            possible_recipes=[]
            possible_recipes = matches["partial"]
            temp_recipes = []
            if possible_recipes:
                for recipe in possible_recipes:
                    try:
                        is_fav = recipe["stars"][uid] != None
                        recipe["user_saved"] = is_fav
                    except KeyError:
                        pass
                    temp_recipes.append(recipe)
                possible_recipes = temp_recipes
        else:
            disp_partial = False
            recipes.set_fridge_recipes(None)

    if(fridge_recipes!="part" and recipes.get_fridge_sorting_type()[:3] == "mis"):
        recipes.set_fridge_sorting_type("name_A")
    possible_recipes = sort_recipes(possible_recipes, recipes.get_fridge_sorting_type()) or possible_recipes


    if(sorting_type:=request.POST.get('sorting')):
            recipes.set_sorting_type(sorting_type)

    possible_recipes = sort_recipes(possible_recipes, recipes.get_sorting_type()) or possible_recipes

    scrollTop = 0
    keep_scroll_pos = False
    found_results = False
    if recipes.get_is_recipe_liked():
        scrollTop = recipes.get_recipe_list_position()
        recipes.set_is_recipe_liked(False)
        keep_scroll_pos = True

    if len(possible_recipes) > 0:
        found_results = True
    paginator = Paginator(possible_recipes, 48)
    page = request.GET.get('page')
    if not page:
        page = "1"
    m_category.set_category_page(page)

    try:
        curr_recipes = paginator.page(page)
    except PageNotAnInteger:
        curr_recipes = paginator.page(1)
    except EmptyPage:
        curr_recipes = paginator.page(paginator.num_pages)
    data = {
        "user": m_user._getUser_(),
        "recipes": curr_recipes,
        "partial": disp_partial,
        "scrollTop": scrollTop,
        "keep_scroll_pos": keep_scroll_pos,
        "found_results": found_results,
        "items": len(possible_recipes),
        "isSearch": True,
        "fridge_recipes" : fridge_recipes,
        "sorting_type": recipes.get_fridge_sorting_type(),
    }

    return render(request, 'fridge_recipes.html', {"data": data})



"""cpanel URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.0/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import path
from . import views
#app_name = 'empty_my_fridge'
#python manage.py runserver
urlpatterns = [
    path('', views.home, name='homepage'),
    path('home/', views.home, name='home'),
    path('recipe_list/', views.recipe_list, name='recipes'),
    path('recipe_page/', views.recipe_page, name='recipe_page'),
    path('login/', views.login, name='login'),
    path('register/', views.register, name='register'),
    path('to_home/', views._login_, name='to_home'),
    path('to_signIn/', views._register_, name='_register_'),
    path('profile/', views.profile, name='profile'),
    path('edit_profile/', views.edit_profile, name='edit_profile'),
    path('save_profile/', views.save_profile, name='save_profile'),
    path('personal_recipes/', views.personal_recipes, name='personal_recipes'),
    path('recipe_view_page/', views.recipe_view_page, name='recipe_view_page'),
    path('friends/', views.user_friends, name='friends'),
    path('public_friends/', views.public_friends, name='public_friends'),
    path('public_profile/', views.public_profile, name='public_profile'),
    path('public_fav_recipes/', views.public_fav_recipes, name='public_fav_recipes'),
    path('public_personal_recipes/', views.public_personal_recipes, name='public_personal_recipes'),
    path('friend_requests/', views.friend_requests, name='friend_requests'),
    path('account_settings/', views.account_settings, name='account_settings'),
    path('recover_password/', views.recover_password, name='recover_password'),
    path('favorite_recipes/', views.user_fav_recipes, name='favorite_recipes'),
    path('fav_recipe_onclick/', views.fav_recipe_onClick, name='fav_recipe_onclick'),
    path('search_and_filter/', views.search, name='search_and_filter'),
    path('scrape_page/', views.scrape_page, name="scrape_page"),
    path('logout/', views._logout_, name='logout'),
    path('categories/', views.category, name='category'),
    path('reset_password/', views.reset_password, name='reset_password'),
    path('upload_image/', views.upload_image, name='upload_image'),
    path('search/', views.get_recipes_by_category_ingredients, name='search'),
    path('fridge/',views.fridge, name='fridge'),
    path('fridge/recipes',views.fridge_recipes, name='fridge recipes'),

]



class Message:
    def __init__(self):
        self.message = None
        self.msg_type = None

    def get_message(self):
        return self.message

    def set_message(self, message):
        self.message = message

    def get_msg_type(self):
        return self.msg_type

    def set_msg_type(self, msg_type):
        self.msg_type = msg_type          



class Recipes:
    def __init__(self):
        self.recipe_list = None
        self.pos = 0
        self.liked = False
        self.searched = False
        self.recipe_name_to_find = None
        self.recipes_current_page = "1"
        self.db = None
        self.m_user = None
        self.food_network = None
        self.visited_pages = ""
        self.scraped = False
        self.filter_list = None
        self.sorting_type = "name_A"
        self.fridge_recipes = None
        self.is_fridge = False
        self.fridge_sorting_type = "name_A"

        

    def __init__(self, db, m_user, food_network):
        self.recipe_list = None
        self.pos = 0
        self.liked = False
        self.searched = False
        self.recipe_name_to_find = None
        self.recipes_current_page = "1"
        self.db = db
        self.m_user = m_user
        self.food_network = food_network
        self.visited_pages = ""
        self.scraped = False
        self.sorting_type = "name_A"
        self.filter_list = None
        self.fridge_recipes = None
        self.is_fridge = False
        self.fridge_sorting_type = "name_A"

    def get_fridge_recipes(self):
        return self.fridge_recipes

    def set_fridge_recipes(self, fridge_recipes):
        self.fridge_recipes = fridge_recipes

    def get_is_fridge(self):
        return self.is_fridge

    def set_is_fridge(self, is_fridge):
        self.is_fridge = is_fridge

    def get_sorting_type(self):
        return self.sorting_type

    def set_sorting_type(self, type):
        self.sorting_type = type

    def get_fridge_sorting_type(self):
        return self.fridge_sorting_type

    def set_fridge_sorting_type(self, type):
        self.fridge_sorting_type = type
    
    def set_scraped(self, scraped):
        self.scraped = scraped

    def get_scraped(self):
        return self.scraped

    def set_visited_pages(self, page):
        self.visited_pages += "," + page

    def get_visited_pages(self, page):
        return self.visited_pages.find(page)

    def get_all_recipes(self):
        return self.recipe_list

    def set_all_recipes(self, recipe_list):
        self.recipe_list = recipe_list

    def set_recipe_list_position(self, pos):
        self.pos = pos

    def get_recipe_list_position(self):
        return self.pos

    def set_is_recipe_liked(self, liked):
        self.liked = liked

    def get_is_recipe_liked(self):
        return self.liked

    def set_is_searched_for_recipes(self, searched):
        self.searched = searched

    def get_is_searched_for_recipes(self):
        return self.searched

    def set_recipe_name_to_find(self, word):
        self.recipe_name_to_find = word

    def get_recipe_name_to_find(self):
        return self.recipe_name_to_find

    def set_recipes_current_page(self, page):
        self.recipes_current_page = page

    def get_recipes_current_page(self):
        return self.recipes_current_page

    def set_recipe_liked(self, key):
        for recipe in self.recipe_list:
            if recipe["recipe_id"] == key:
                recipe["user_saved"] = True
                recipe["likes"] = recipe["likes"] + 1
                break

    def set_recipe_unLiked(self, key):
        for recipe in self.recipe_list:
            if recipe["recipe_id"] == key:
                recipe["user_saved"] = False
                recipe["likes"] = recipe["likes"] - 1
                break                                     

    def get_all_likes(self, uid, page):
        page_num = int(page)
        start = (page_num - 1) * 48
        favorite = False
        
        while start < len(self.recipe_list):
            recipe = self.recipe_list[start]
            recipe["no_user_signed_in"] = True
            if start == len(self.recipe_list) or start == page_num * 48:
                break
            try:
                key = recipe["recipe_id"]
                if uid:
                    recipe["no_user_signed_in"] = False
                    favorite = recipe["stars"][uid] != None
                    recipe["user_saved"] = favorite
            except KeyError:
                pass
            start+=1
             
    # get all recipes
    def _get_all_recipes_(self):
        #self.db.child('all_ingredients').remove()
        #self.food_network.food_network(self.db)
        all_recipes = self.db.child("recipe").get()

        recipe_list = []
        if all_recipes.each() != None:
            for recipe in all_recipes.each():
                key = str(recipe.key())
                try:
                    name = recipe.val()["recipe_name"]
                    _recipe_ = self.get_recipe(dict(recipe.val()), key, self.m_user._getUser_Id_())
                    recipe_list.append(_recipe_)
                except KeyError:
                    self.db.child("recipe").child(key).remove()
                    continue


        self.recipe_list = recipe_list  

    # get individual recipe as Json
    def get_recipe(self, recipe, key, uid):
        num_of_stars = 0
        favorite = False
        recipe["recipe_id"] = key
        recipe["no_user_signed_in"] = True
        try:
            if uid:
                recipe["no_user_signed_in"] = False
                favorite = recipe["stars"][uid] != None
            recipe["user_saved"] = favorite
        except KeyError:
            recipe["user_saved"] = False
            pass
        try:
            num_of_stars = len(recipe["stars"].items())
            recipe["likes"] = num_of_stars
        except KeyError:
            recipe["likes"] = 0
            pass    

        return recipe
    
    def set_filter_list(self, filters):
        if not filters:
            self.filter_list = None
        elif len(filters) == 0:
            self.filter_list = None
        else:
            self.filter_list = filters

    def get_filter_list(self):
        return self.filter_list



def myConfig():
    config = {
        #becca"s firebase config
        "apiKey": "AIzaSyDMlZ3DzGdgJ7aQc3iUl-S5lG6GpWMhCOw",
        "authDomain": "empty-my-fridge-6197d.firebaseapp.com",
        "databaseURL": "https://empty-my-fridge-6197d.firebaseio.com",
        "projectId": "empty-my-fridge-6197d",
        "storageBucket": "empty-my-fridge-6197d.appspot.com",
        "messagingSenderId": "210802633597",
        "appId": "1:210802633597:web:239240de47cbf5e2b6ba14",
        "measurementId": "G-26T7SBKJCX"

        #edward"s firebase config
        """
        "apiKey": "AIzaSyBUbHTvIh5bIb804EzPpHThsItT6i51TjY",
        "authDomain": "empty-my-fridge.firebaseapp.com",
        "databaseURL": "https://empty-my-fridge.firebaseio.com",
        "projectId": "empty-my-fridge",
        "storageBucket": "empty-my-fridge.appspot.com",
        "messagingSenderId": "394055887941",
        "appId": "1:394055887941:web:d0fa955898ec86e077872c",
        "measurementId": "G-38YRH5Z55V"
        """
        
        """
        "apiKey": "AIzaSyBkIAGYXQ-yVBkoRqXKuBKGuXPUpNzAP7g",
        "authDomain": "empty-my-fridge-ff73c.firebaseapp.com",
        "databaseURL": "https://empty-my-fridge-ff73c.firebaseio.com",
        "projectId": "empty-my-fridge-ff73c",
        "storageBucket": "empty-my-fridge-ff73c.appspot.com",
        "messagingSenderId": "904741248232",
        "appId": "1:904741248232:web:33ba3b5e81be538db2fd5d",
        "measurementId": "G-9SLJ34LP0M"
        """  
    }
    return config



from urllib.request import urlopen as uReq
from bs4 import BeautifulSoup as soup
from builtins import any as b_any
import time
import unicodedata

# list of measurement units for parsing ingredient
measurementUnits = ['teaspoons','tablespoons','cups','containers','packets','bags','quarts','pounds','cans','bottles',
		'pints','packages','ounces','jars','heads','gallons','drops','envelopes','bars','boxes','pinches',
		'dashes','bunches','recipes','layers','slices','links','bulbs','stalks','squares','sprigs',
		'fillets','pieces','legs','thighs','cubes','granules','strips','trays','leaves','loaves','halves', 'cloves', 'large', 
		'extra-large', 'small', 'grams','milliliters', 'sticks', 'whole', 'handful', 'unsalted', 'salted', 'baby']

measurementUnitsAbbreviations = ['c', 'c.', 'C','gal', 'oz', 'oz.', 'pt', 'pts', 'pt.', 'lb', 'lb.', 'lbs', 'lbs.', 'Lb', 'Lbs', 'qt', 
		'qt.', 'qts', 'qts.', 'tbs','tbs.', 'tbsp', 'tbsp.', 'tbspn','tbspn.', 'T', 'T.','tsp','tsp.', 'tspn','tspn.', 't', 't.','g', 'g.', 'kg', 'kg.', 
		'Kg', 'Kg.', 'l', 'l.', 'L', 'L.', 'mg', 'mg.', 'ml', 'ml.', 'mL', 'mL.', 'pkg', 'pkgs', 'pcs', 'pcs.', ]

# list of adjectives and participles used to describe ingredients
descriptions = ['baked', 'beaten', 'blanched', 'boiled', 'boiling', 'boned', 'breaded', 'brewed', 'broken', 'chilled',
		'chopped', 'cleaned', 'coarse', 'cold', 'cooked', 'cool', 'cooled', 'cored', 'creamed', 'crisp', 'crumbled',
		'crushed', 'cubed', 'cut', 'deboned', 'deseeded', 'diced', 'dissolved', 'divided', 'drained', 'dried', 'dry',
		'fine', 'firm', 'fluid', 'fresh', 'frozen', 'grated', 'grilled', 'ground', 'halved', 'hard', 'hardened',
		'heated', 'heavy', 'juiced', 'julienned', 'jumbo', 'large', 'lean', 'light', 'lukewarm', 'marinated',
		'mashed', 'medium', 'melted', 'minced', 'near', 'opened', 'optional', 'packed', 'peeled', 'pitted', 'popped',
		'pounded', 'prepared', 'pressed', 'pureed', 'quartered', 'refrigerated', 'rinsed', 'ripe', 'roasted',
		'roasted', 'rolled', 'rough', 'scalded', 'scrubbed', 'seasoned', 'seeded', 'segmented', 'separated',
		'shredded', 'sifted', 'skinless', 'sliced', 'slight', 'slivered', 'small', 'soaked', 'soft', 'softened',
		'split', 'squeezed', 'stemmed', 'stewed', 'stiff', 'strained', 'strong', 'thawed', 'thick', 'thin', 'tied', 
		'toasted', 'torn', 'trimmed', 'wrapped', 'vained', 'warm', 'washed', 'weak', 'zested', 'wedged',
		'skinned', 'gutted', 'browned', 'patted', 'raw', 'flaked', 'deveined', 'shelled', 'shucked', 'crumbs',
		'halves', 'squares', 'zest', 'peel', 'uncooked', 'butterflied', 'unwrapped', 'unbaked', 'warmed', 'cracked','good','store', 
		'bought', 'fajita-sized', 'finely', 'freshly','slow', 'quality', 'sodium', 'mixed', 'wild', 'French', 'African' ,'Mexican','Asian', 'Italian', 'Chinese', 'American', 
		'garnished', 'seedless','coarsely', 'natural', 'organic', 'solid','solid', 'heaping','stoned', 'homemade', 'canned', 'unsweetened', 'instant','overripe']

# list of common ingredients that accidentally get filtered out due to similarities in description list
description_exceptions = ['butter', 'oil', 'cream', 'bread','all', 'salt', 'sour']

nonplurals = ['eggs', 'sugars', 'whites', 'tortillas', 'peppers','carrots', 'limes', 'pecans','zucchinis', 'apples']
# list of numbers as words
numbers = ['one', 'two','three','four','five','six','seven','eight','nine','ten', 'elevin','twelve','dozen']

brands = ['bertolli®', 'cook\'s', 'hothouse', 'NESTLÉ®', 'TOLL HOUSE®']

# misc modifiers (Will sort later)
modifier = ['plus', 'silvered', 'virgin', 'seasoning', 'taste', 'yolks', 'meat', 'frying', 'dusting']

# list of adverbs used before or after description
precedingAdverbs = ['well', 'very', 'super']
succeedingAdverbs = ['diagonally', 'lengthwise', 'overnight']

# list of prepositions used after ingredient name
prepositions = ['as', 'such', 'for', 'with', 'without', 'if', 'about', 'e.g.', 'in', 'into', 'at', 'until', 'won\'t']

# only used as <something> removed, <something> reserved, <x> inches, <x> old, <some> temperature
descriptionsWithPredecessor = ['removed', 'discarded', 'reserved', 'included', 'inch', 'inches', 'old', 'temperature', 'up', ]

# descriptions that can be removed from ingredient, i.e. candied pineapple chunks
unnecessaryDescriptions = ['chunks', 'pieces', 'rings', 'spears', 'style', 'desserts']

# list of prefixes and suffixes that should be hyphenated
hypenatedPrefixes = ['non', 'reduced', 'semi', 'low']
hypenatedSuffixes = ['coated', 'free', 'flavored']

vulgarFractions = ['¼','½','¾','⅐','⅑','⅒','⅓','⅔','⅕','⅖','⅗','⅘','⅙','⅚','⅛','⅜','⅝','⅞','⅟']

def parser(ingredient, food_array):
	if type(ingredient) != str:
	   return 	
	
	parsed_word = ''
	
	# Removes unnecessary special characters
	ingredient = ingredient.strip()
	ingredient = ingredient.replace('-', ' ')
	ingredient = ingredient.replace('+', ' ')
	ingredient = ingredient.replace(':', ' ')
	ingredient = ingredient.replace(';', ' ')
	ingredient = ingredient.replace('/', ' ')
	ingredient = ingredient.replace('\'', ' ')
	ingredient = ingredient.replace('\"', ' ')
	ingredient = ingredient.replace('%', ' ')
	ingredient = ingredient.replace('.', ' ')
	ingredient = ingredient.replace('&', ' ')
	ingredient = ingredient.replace('[', ' ')
	ingredient = ingredient.replace(']', ' ')
	ingredient = ingredient.replace('®', '')
	ingredient = ingredient.replace('\u2009',' ')
	# Breaks each word into a string array
	split_item = ingredient.split(" ")
	# print(split_item)
	for word in split_item:
		word = word.lower()
		#Takes care of wholenumbers, decimals, and fractions
		if word.isnumeric() or word.isdecimal():
			continue
		elif b_any(word in x for x in description_exceptions):
			parsed_word = word + ' '
			continue
		elif word in nonplurals:
			word = word[:-1]
			parsed_word = parsed_word + word + ' '
			continue	
		elif ',' in word:
			last_word = word.replace(',','')
			if last_word in nonplurals:
				last_word = last_word[:-1]
			parsed_word = parsed_word + last_word
			break
		elif word == 'or':
			break
		elif word == 'and':
			parsed_word = parsed_word.rstrip()
			# food_array.append(parsed_word)
			parsed_word = ''
			continue
		elif '(' in word or ')' in word:
			continue
		elif b_any(word in x for x in vulgarFractions):
			continue
		elif b_any(word in x for x in measurementUnits):
			continue
		elif b_any(word in x for x in measurementUnitsAbbreviations):
			continue
		elif b_any(word in x for x in numbers):
			continue
		elif b_any(word in x for x in brands):
			continue
		elif b_any(word in x for x in descriptions):
			continue
		elif b_any(word in x for x in modifier):
			continue
		elif b_any(word in x for x in precedingAdverbs):
			continue
		elif b_any(word in x for x in succeedingAdverbs):
			continue
		elif b_any(word in x for x in prepositions):
			continue
		elif b_any(word in x for x in descriptionsWithPredecessor):
			continue
		elif b_any(word in x for x in unnecessaryDescriptions):
			continue
		elif b_any(word in x for x in hypenatedPrefixes):
			continue	
		elif b_any(word in x for x in hypenatedSuffixes):
			continue			
		else: 
			parsed_word = parsed_word + word + ' '
	
	parsed_word = parsed_word.strip()
	#return parsed_word
	#print(parsed_word)
	#Prevent's blank spots in ingredients array
	if parsed_word == '':
		return food_array
	else:
		food_array.append(parsed_word)
		return food_array
	
	
def allrecipes(db):
	menu = "Stop scraping after...page numbers completed.\n1. 3 pages\n2. 5 pages\n3. All pages\n4. Custom\nOption: "
	option = input(menu)
	num = 0
	page_num = 0
	if option == "1":
		num = 0
		page_num = 3
	elif option == "2":
		num = 0
		page_num = 5
	elif option == "3":
		num = 0
		page_num = 200
	elif option == "4":
		num = input("Set start page: ")
		num = int(num)
		page_num = input("Set end page: ")
		page_num = int(page_num)

	print('Getting all ingredients in database. Please wait....')
	
	_all_ingredients_ = db.child("all_ingredients").get().val()
	print('Done!')
	print('\n')	
	#Array that will hold the info for each individual recipe
	grand_recipe_list = [] 
	print('Now scraping for recipes!')
	print('For the time being, get yourself a coffee while you wait.')
	print('\n')

	while num <= page_num:
		print('Now scraping page ' + str(num) + ' out of ' + str(page_num))
		# ---------- ---------- Scraping Page With All Recipe Links---------- ----------
		allrecipe = 'https://www.allrecipes.com/recipes/?page=' + str(num)
		uClient = uReq(allrecipe)
		page_html = uClient.read()
		uClient.close()
		page_soup = soup(page_html, "html.parser")
		recipe_cards = page_soup.findAll('article', {"class":"fixed-recipe-card"}) #The area where the recipe cards are
		num = num + 1
		# ---------- ---------- Scraping Each Recipe Link In Page---------- ----------
		for recipe in recipe_cards:
			#Array Layout order as follows: [Title, Image, Link to Recipe, Ingredients, Categories]
			add_to_grand_list = []   
			
			recipe_link = recipe.a.get('href')
			uClient = uReq(recipe_link)
			page_html = uClient.read()
			uClient.close()
			page_soup = soup(page_html, "html.parser")
			
			#gets relavent html info (2 sets of html elements due to having a modern and legacy page layout options)
			
			if page_soup.find("h1", {"class":"headline heading-content"}) != None:
				recipe_title = page_soup.find("h1", {"class":"headline heading-content"})
			else:
				recipe_title = page_soup.find("h1", {"class":"recipe-summary__h1"})

			if page_soup.find("div", {"class":"image-container"}) != None:
				recipe_img = page_soup.find("div", {"class":"image-container"})
			else:
				recipe_img = page_soup.find("img", {"class":"rec-photo"})

			if page_soup.findAll("span", {"class":"ingredients-item-name"}) != []:
				recipe_ingredients = page_soup.findAll("span", {"class":"ingredients-item-name"})
			else:
				recipe_ingredients = page_soup.findAll("span", {"class":"recipe-ingred_txt added"})

			if page_soup.findAll('span', {"class":"breadcrumbs__title"}) != []:
				recipe_cat = page_soup.findAll('span', {"class":"breadcrumbs__title"})
			else:
				# recipe_cat = page_soup.findAll('meta', itemprop="recipeCategory") Gets categories from meta data, may use later
				recipe_cat = page_soup.findAll('span', {"class": "toggle-similar__title"})

			# ---------- ---------- Getting Title ---------- ----------
			if recipe_title != None:
				add_to_grand_list.append(recipe_title.text)
			else:
				add_to_grand_list.append('No Title Avaliable')

			# ---------- ---------- Getting Image Source ---------- ----------
			if recipe_img.div != None:
				add_to_grand_list.append(recipe_img.div['data-src']) #Image from modern page
			elif recipe_img != None:
				add_to_grand_list.append(recipe_img['src']) # Image from legacy page
			else:
				add_to_grand_list.append('https://cdn.dribbble.com/users/1012566/screenshots/4187820/topic-2.jpg') # Placeholder image if no image is avaliable

			# ---------- ---------- Getting Recipe Source ---------- ----------
			add_to_grand_list.append(recipe_link)
			print(recipe_link)
			# ---------- ---------- Getting Ingredients ---------- ----------
			ingredient_list = []
			if recipe_ingredients != None:
				for ingredient in recipe_ingredients:			
					ingredient_list = parser(ingredient.text, ingredient_list)
			add_to_grand_list.append(ingredient_list)

			
			# ---------- ---------- Getting Recipe Categories ---------- ----------
			category_list = []
			if recipe_cat != None:
				recipe_script_pin = page_soup.find("script", {"id":"karma-loader"})
				recipe_script_legacy = page_soup.find_all("script")

				if recipe_script_pin != None:
					step_0 = str(recipe_script_pin)
					step_1 = step_0.split('tags: [')
					step_2 = " ".join(step_1[1].split())
					step_3 = step_2.split(']')[0].strip()
					step_4 = step_3.replace('"', '')
					step_5 = step_4.split(',')
					for category in step_5:
						category = category.strip()
						category_list.append(category.lower())
				elif recipe_script_legacy != None:
					try:
						step_0 = str(recipe_script_legacy[30])
						step_1 = step_0.split('var RdpInferredTastePrefs = [')
						step_2 = " ".join(step_1[1].split())
						step_3 = step_2.split(']')[0].strip()
						step_4 = step_3.replace('"', '')
						step_5 = step_4.split(',')
						for category in step_5:
							category = category.strip()
							category_list.append(category.lower())
					except IndexError:
						category_list.append("None")
						pass
				else:	
					for category in recipe_cat:
						category = category.text.strip()
						if category == 'Home': # Not a category
							continue
						elif category == 'Chevron Right': # Weird '<' character
							continue
						elif category == 'Recipes': # Not a category
							continue
						else:
							category_list.append(category.lower())
			add_to_grand_list.append(category_list)

			grand_recipe_list.append(add_to_grand_list)
			#Used to have a 1 second delay for each recipe scraped. Helps prevents forced connection drops from host
			time.sleep(0.5)
		
		res = '{0} recipes have been scraped.\nStopping...Please wait...'.format((num - 1) * 21)
		if page_num == num - 1:
			print(res)
			break
			


	#[Title, Image, Link to Recipe, Ingredients array]
	print("Populating database. please wait...")
	if not _all_ingredients_:
		_all_ingredients_ = []
	for recipe in grand_recipe_list:
		#Checks to see if list is empty (Will not inclide recipe_ing)
		if not recipe[3]:
			ingredients = ['No ingredients']
		else:
			ingredients = recipe[3]
		recipe_name = recipe[0]
		recipe = {
		'recipe_name': recipe_name,
		'recipe_image': recipe[1],
		'recipe_link': recipe[2],
		'recipe_ingredients': ingredients,
		'recipe_categories': recipe[4],
		}
		#db.child('recipe').push(recipe)		
		
		found = False

		#if ingredients not in _all_ingredients_:
		_all_ingredients_ = list(dict.fromkeys(_all_ingredients_ + ingredients))
		all_recipes = db.child('recipe').get().each()
		if all_recipes != None:
			for m_recipe in all_recipes:
				_recipe_ = m_recipe.val()
				try:
					if _recipe_["recipe_ingredients"] == ingredients and _recipe_["recipe_name"] == recipe_name:
						found = True
						break
				except KeyError:
					pass	
		
		if not found:
			db.child('recipe').push(recipe)
			
	db.child('all_ingredients').set(_all_ingredients_)
	print("Done!\nDb has been populated with new data")	



#imports packages  (Be sure to install BeautifulSoup)
from urllib.request import urlopen as uReq
from bs4 import BeautifulSoup as soup
from builtins import any as b_any
from ctypes import *
import time
import os
import sys, select


#17 Sample recipes for testing (Provided by foodnetwork.com)
# recipes = ['https://www.foodnetwork.com/recipes/food-network-kitchen/senate-bean-soup-recipe-1973240', 'https://www.foodnetwork.com/recipes/food-network-kitchen/applesauce-waffles-3362190', 'https://www.foodnetwork.com/recipes/food-network-kitchen/spaghetti-with-oil-and-garlic-aglio-et-olio-recipe-1944538', 'https://www.foodnetwork.com/recipes/food-network-kitchen/spaghetti-cacio-e-pepe-3565584', 'https://www.foodnetwork.com/recipes/ina-garten/cinnamon-baked-doughnuts-recipe-2135621', 'https://www.foodnetwork.com/recipes/food-network-kitchen/pancakes-recipe-1913844', 'https://www.foodnetwork.com/recipes/alton-brown/granola-recipe-1939521', 'https://www.foodnetwork.com/recipes/food-network-kitchen/healthy-banana-bread-muffins-recipe-7217371', 'https://www.foodnetwork.com/recipes/chocolate-lava-cakes-2312421', 'https://www.foodnetwork.com/recipes/ina-garten/garlic-roasted-potatoes-recipe-1913067', 'https://www.foodnetwork.com/recipes/robert-irvine/french-toast-recipe-1951408','https://www.foodnetwork.com/recipes/food-network-kitchen/curry-fried-rice-recipe-2109760', 'https://www.foodnetwork.com/recipes/ree-drummond/beef-tacos-2632842','https://www.foodnetwork.com/recipes/food-network-kitchen/sweet-and-sour-couscous-stuffed-peppers-recipe-2121036','https://www.foodnetwork.com/recipes/dave-lieberman/mexican-chicken-stew-recipe-1917174','https://www.foodnetwork.com/recipes/food-network-kitchen/cauliflower-gnocchi-4610559','https://www.foodnetwork.com/recipes/sunny-anderson/easy-chicken-pot-pie-recipe-1923875']

#Food Network recipe list from A-Z (Warning: This will take a really long time to scrape)
network_recipes = ['https://www.foodnetwork.com/recipes/recipes-a-z/', 'https://www.foodnetwork.com/recipes/recipes-a-z/a', 'https://www.foodnetwork.com/recipes/recipes-a-z/b', 'https://www.foodnetwork.com/recipes/recipes-a-z/c','https://www.foodnetwork.com/recipes/recipes-a-z/d','https://www.foodnetwork.com/recipes/recipes-a-z/e','https://www.foodnetwork.com/recipes/recipes-a-z/f','https://www.foodnetwork.com/recipes/recipes-a-z/g','https://www.foodnetwork.com/recipes/recipes-a-z/h','https://www.foodnetwork.com/recipes/recipes-a-z/i','https://www.foodnetwork.com/recipes/recipes-a-z/j','https://www.foodnetwork.com/recipes/recipes-a-z/k','https://www.foodnetwork.com/recipes/recipes-a-z/l','https://www.foodnetwork.com/recipes/recipes-a-z/m','https://www.foodnetwork.com/recipes/recipes-a-z/n','https://www.foodnetwork.com/recipes/recipes-a-z/o','https://www.foodnetwork.com/recipes/recipes-a-z/p','https://www.foodnetwork.com/recipes/recipes-a-z/q','https://www.foodnetwork.com/recipes/recipes-a-z/r','https://www.foodnetwork.com/recipes/recipes-a-z/s','https://www.foodnetwork.com/recipes/recipes-a-z/t','https://www.foodnetwork.com/recipes/recipes-a-z/u','https://www.foodnetwork.com/recipes/recipes-a-z/v','https://www.foodnetwork.com/recipes/recipes-a-z/w','https://www.foodnetwork.com/recipes/recipes-a-z/xyz',]

#network_recipes = ['https://www.foodnetwork.com/recipes/recipes-a-z/b']

# recipes = ['https://www.foodnetwork.com/recipes/food-network-kitchen/senate-bean-soup-recipe-1973240', 'https://www.foodnetwork.com/recipes/farfalle-with-herb-marinated-grilled-shrimp-2118008', 'https://www.foodnetwork.com/recipes/farfalle-al-rocco-recipe-1909940', 'https://www.foodnetwork.com/recipes/giada-de-laurentiis/farfalle-with-broccoli-recipe-1945696', 'https://www.foodnetwork.com/recipes/jamie-oliver/farfalle-with-savoy-cabbage-pancetta-thyme-and-mozzarella-recipe2-1909322', 'https://www.foodnetwork.com/recipes/guy-fieri/far-out-farro-salad-recipe-2108223']

app_path = os.path.abspath(__file__)
app_path = os.path.realpath(app_path)
app_path = os.path.dirname(app_path)

if os.name == "nt":
	app_path = os.path.join(app_path, "recipe_parsing.dll")
	lib = CDLL(app_path)
else:
	app_path = os.path.join(app_path, "librecipe_parsing.so")
	lib = CDLL(app_path)

lib.parser.restype = c_char_p
lib.parser.argtype = c_char_p

#Parses through the ingredient list
def minor_parsing(food):
	food = str(food)
	food = food[1:]
	food = food.replace('\xc2\xa0','')
	food = food.replace('\'', '')
	food = food.strip()
	return food


def food_network(db):
	print('Getting all recipes in database. Please wait....')
	
	# should be a total of 24 pages
	page_num = 0
	all_recipes = db.child('recipe').get().each()
	"""
	_all_recipes_ = []
	if _all_recipes_.each() != None:
		for m_recipe in _all_recipes_.each():
			_recipe_ = m_recipe.val()
	"""
			
	
	print('Done!')
	print('\n')		
	print('Now scraping for recipes!')
	print('For the time being, get yourself a coffee while you wait.')
	print('\n')
	
	#Holds an array of recipe info, the individual array in the grand array is arranged in this order --> [Title, Image, Link to Recipe, Ingredients array]
	grand_recipe_list = []
	for page_link in network_recipes:
		print('Now scraping page ' + str(page_num) + ' out of 24' )
		page_num = page_num + 1
		# ---------- ---------- Scraping Page With All Recipe Links---------- ----------
		recipe_link = page_link
		uClient = uReq(recipe_link)
		page_html = uClient.read()
		uClient.close()
		page_soup = soup(page_html, "html.parser")
		recipes = page_soup.findAll('li', {"class":"m-PromoList__a-ListItem"})

		cap = 0
		# ---------- ---------- Scraping Each Recipe Link In Page---------- ----------
		for recipe in recipes:
			if cap == 10:
				break
			the_recipe = str(recipe.find('a')['href'])
			the_recipe = the_recipe.replace('//', '')
			the_recipe = 'https://' + the_recipe
			print(the_recipe)
			
			
			#Website im scraping info on (Default homepage) 
			#recipe_page = the_recipe
			#Essentially opening up the connection and downloads the whole html webpage
			uClient = uReq(the_recipe)
			page_html = uClient.read()
			uClient.close()

			#Array that will hold the info for each individual recipe
			add_to_grand_list = []

			#Parses the html data (This is where the fun begins)
			page_soup = soup(page_html, "html.parser")

			#gets relavent html info
			recipe_title = page_soup.find("span", {"class":"o-AssetTitle__a-HeadlineText"})
			recipe_img = page_soup.find("img", {"class":"m-MediaBlock__a-Image a-Image"})
			recipe_ingredients = page_soup.findAll("p", {"class":"o-Ingredients__a-Ingredient"})
			recipe_cat = page_soup.findAll('a', {"class":"o-Capsule__a-Tag a-Tag"})

			
			# ---------- ---------- Getting Title ---------- ----------
			if recipe_title != None:
				add_to_grand_list.append(recipe_title.text)
			else:
				add_to_grand_list.append('No Title')
			
			# ---------- ---------- Getting Image Source ---------- ----------
			if recipe_img.get("src") != None:
				add_to_grand_list.append(str(recipe_img.get("src")))
			else:
				add_to_grand_list.append('https://cdn.dribbble.com/users/1012566/screenshots/4187820/topic-2.jpg')
			# ---------- ---------- Getting Recipe Source ---------- ----------
			add_to_grand_list.append(the_recipe)
			
			# ---------- ---------- Getting Ingredients ---------- ----------
			ingredient_list = []
			if recipe_ingredients != None:
				for item in recipe_ingredients:			
					bite = bytes(item.text, 'utf-8')
					food = lib.parser(bite)
					food = minor_parsing(food)
					#print(item.text)
					#print(food)
					ingredient_list.append(food)
			add_to_grand_list.append(ingredient_list)

			
			# ---------- ---------- Getting Recipe Categories ---------- ----------
			category_list = []
			if recipe_cat != None:
				for cat in recipe_cat:
					#print(cat.text)
					category_list.append(cat.text.lower())
			add_to_grand_list.append(category_list)

			grand_recipe_list.append(add_to_grand_list)

			#Used to have a 1 second delay for each recipe scraped. Helps prevents forced connection drops from host
			time.sleep(1)
			cap += 1
			
		if page_num % 5 == 0:
			prompt = 'Idle: Scraping paused...\n{0} recipes have been scraped.\n*Note: If you continue with no, scraped recipes would be populated in database\nResume scraping? y/N: '.format((page_num - 1) * 10)
			c = input(prompt)
			if c.lower() == 'n':
				print("Stopping...Please wait...")
				break;
		
	
	#[Title, Image, Link to Recipe, Ingredients array]
	print("Populating database. please wait...")
	all_categories = []
	_all_ingredients_ = []
	
	for recipe in grand_recipe_list:
		#Checks to see if list is empty (Will not inclide recipe_ing)
		if not recipe[3]:
			ingredients = ['No ingredients']
		else:
			ingredients = recipe[3]
		recipe_name = recipe[0]
		categories = recipe[4]
		recipe = {
		'recipe_name': recipe_name,
		'recipe_image': recipe[1],
		'recipe_link': recipe[2],
		'recipe_ingredients': ingredients,
		'recipe_categories': categories,
		}		
		
		found = False
		_all_ingredients_ = list(dict.fromkeys(_all_ingredients_ + ingredients))
		all_categories = list(dict.fromkeys(all_categories + categories))
		
		if all_recipes != None:
			for m_recipe in all_recipes:
				_recipe_ = m_recipe.val()
				try:
					if _recipe_["recipe_ingredients"] == ingredients and _recipe_["recipe_name"] == recipe_name:
						found = True
						break
				except KeyError:
					pass	
		
		if not found:
			db.child('recipe').push(recipe)
				
	
	db.child('all_ingredients').set(_all_ingredients_)
	db.child('all_categories').set(all_categories)
	print("Done!\nDb has been populated with new data")
				


#!/usr/bin/env python
"""Django's command-line utility for administrative tasks."""
import os
import sys
settings = 'empty_my_fridge.cpanel.settings'
try:
    import empty_my_fridge.manage
except ModuleNotFoundError:
    settings = 'cpanel.settings'

def main():
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', settings)
    try:
        #from django.core.management import execute_from_command_line
        from django.core.management import call_command
        from django.core.wsgi import get_wsgi_application
    except ImportError as exc:
        raise ImportError(
            "Couldn't import Django. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        ) from exc
    #execute_from_command_line(sys.argv)
    application = get_wsgi_application()
    call_command('runserver', '127.0.0.1:8000')
    


if __name__ == '__main__':
    main()
   



class Category:
    def __init__(self):
        self.category = None
        self.category_page = "1"
        self.is_cat_and_ingrd = False
        self.filter_list = None
        self.sorting_type = "name_A"

    def get_sorting_type(self):
        return self.sorting_type

    def set_sorting_type(self, type):
        self.sorting_type = type

    def set_category(self, category):
        self.category = category

    def get_category(self):
        return self.category

    def set_category_page(self, page):
        self.category_page = page

    def get_category_page(self):
        return self.category_page

    def set_cat_and_ingrd(self, is_cat_and_ingrd):
        self.is_cat_and_ingrd = is_cat_and_ingrd

    def get_cat_and_ingrd(self):
        return self.is_cat_and_ingrd

    def set_filter_list(self, filters):
        if not filters:
            self.filter_list = None
        elif len(filters) == 0:
            self.filter_list = None
        else:
            self.filter_list = filters

    def get_filter_list(self):
        return self.filter_list


import os
import setuptools
with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='empty_my_fridge',
    version="1.0.9",
    author="Charles Charlestin, Rebecca Boes, Cyan Perez, Randolph Maynes, Edward Mensah",
    description="Get your Daily Recipes",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/edwarddubi/empty_my_fridge_django",
    packages=setuptools.find_packages(),
    classifiers=[
        "Environment :: Web Environment",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data = True,
    python_requires='>=3.6',
    entry_points = {
        'console_scripts': [
            'empty_my_fridge=empty_my_fridge.manage:main',
        ],
    },
)



class User:
    def __init__(self, user, uid):
        self.user = user
        self.uid = uid
        self.friend_id = None
        #super().__init__()

    def __init__(self):
        self.user = None
        self.uid = None
        self.friend_id = None

    def _getUser_(self):
        return self.user

    def _setUser_(self, user):
        self.user = user

    def _setUser_Id_(self, uid):
        self.uid = uid

    def _getUser_Id_(self):
        return self.uid

    def _getUser_email(self):
        return self.user["email"]

    def _getUser_name(self):
        return self.user["name"]

    def _getUser_username(self):
        return self.user["username"]

    def _getUser_image(self):
        return self.user["image"]

    def _getUser_joined_date(self):
        return self.user["joined"]

    def _getUser_bio(self):
        return self.user["bio"]     

    def _isNone_(self):
        if self.user is None and self.uid is None:
            return True
        return False
        
    def get_friend_id(self):
        return self.friend_id
        
    def set_friend_id(self, id):
        self.friend_id = id


class ActivityPage:
    def __init__(self):
        self.activity = None

    def get_activity_page(self):
        return self.activity

    def set_activity_page(self, activity):
        self.activity = activity       